const tools = require("./tools");

cc.Class({
	extends: cc.Component,

	properties: {
		picNode: {
			type: cc.Node,
			default: null
		},
		picTempNode: {
			type: cc.Node,
			default: null
		},
		leftNode: {
			type: cc.Node,
			default: null
		},
		rightNode: {
			type: cc.Node,
			default: null
		},
		btnClip: cc.AudioClip
	},

	// 生命周期 onLoad
	onLoad() {
		this.initSceneData();
		this.initSize();
	},

	_imageResList: null,
	_audioResList: null,
	_animaResList: null,
	initSceneData() {
		this._imageResList = [];
		this._audioResList = [];
		this._animaResList = [];
	},

	_designSize: null,  // 设计分辨率
	_frameSize: null,  // 屏幕分辨率
	_mapScaleMin: null,  // 场景中常用缩放（取大值）
	_mapScaleMax: null,  // 场景中常用缩放（取小值）
	_cocosScale: null,  // cocos 自缩放 （较少用到）
	initSize() {
		// 注意cc.winSize只有在适配后(修改fitHeight/fitWidth后)才能获取到正确的值,因此使用cc.getFrameSize()来获取初始的屏幕大小
		let screen_size = cc.view.getFrameSize().width / cc.view.getFrameSize().height
		let design_size = cc.Canvas.instance.designResolution.width / cc.Canvas.instance.designResolution.height
		let f = screen_size >= design_size
		cc.Canvas.instance.fitHeight = f
		cc.Canvas.instance.fitWidth = !f

		const frameSize = cc.view.getFrameSize();
		this._frameSize = frameSize;

		this._designSize = cc.view.getDesignResolutionSize();

		let sx = cc.winSize.width / frameSize.width;
		let sy = cc.winSize.height / frameSize.height;
		this._cocosScale = Math.min(sx, sy);

		sx = frameSize.width / this._designSize.width;
		sy = frameSize.height / this._designSize.height;
		this._mapScaleMin = Math.min(sx, sy) * this._cocosScale;
		this._mapScaleMax = Math.max(sx, sy) * this._cocosScale;

		cc.director._scene.width = frameSize.width;
		cc.director._scene.height = frameSize.height;
	},


	// 生命周期 start
	start() {
		let getData = this.getData.bind(this);
		if (window && window.courseware) {
			getData = window.courseware.getData;
		}

		getData((data) => {
			console.log('data:', data);
			this.data = (data && data.length) ? data : this.getDefaultData();
			this.data = JSON.parse(JSON.stringify(this.data))
			this.preloadItem()
		})
		//init show
		this.picNode.opacity = 0
		this.leftNode.active = false
		this.rightNode.active = false
	},

	onEnable() {
		cc.systemEvent.on('playAudio', this.onPlayAudio, this)
	},

	onDisable() {
		cc.systemEvent.off('playAudio', this.onPlayAudio, this)
	},


	getData(cb) {
		cb(this.getDefaultData());
	},

	getDefaultData() {
		const dataJson = '[{"bgItem":{"url":"http://staging-teach.cdn.ireadabc.com/335dea1b83498b0c7c05be637dc13dad.jpeg","rect":{"x":246.27,"y":0,"width":114.46,"height":233},"useBorder":true},"hotZoneItemArr":[{"id":"1620754084744","index":0,"pic_url":"http://staging-teach.cdn.ireadabc.com/335dea1b83498b0c7c05be637dc13dad.jpeg","audio_url":"http://staging-teach.cdn.ireadabc.com/8f73a6b73a217d25ca4f4a2e234c1513.mp3","itemType":"rect","fontScale":0.47421875,"imgScale":0.4640625,"mapScale":0.47421875,"gIdx":"1","useHand":true,"useBorder":true,"rect":{"x":184.73,"y":92,"width":69.01,"height":69.01}},{"id":"1621558296515","index":1,"audio_url":"http://staging-teach.cdn.ireadabc.com/c8a7dd49a778b0b6dd509aac0ef437ff.mp3","itemType":"rect","fontScale":0.47421875,"imgScale":1,"mapScale":0.47421875,"gIdx":"1","useHand":true,"useBorder":true,"rect":{"x":-140.27,"y":99,"width":59,"height":59}}]},{"bgItem":{"url":"http://staging-teach.cdn.ireadabc.com/31bde9b15582618fa5db4ac7cbc275b4.gif","rect":{"x":56.34,"y":0,"width":1005.32,"height":423}},"hotZoneItemArr":[{"id":"1621560079894","index":0,"pic_url":"http://staging-teach.cdn.ireadabc.com/335dea1b83498b0c7c05be637dc13dad.jpeg","audio_url":"http://staging-teach.cdn.ireadabc.com/8f73a6b73a217d25ca4f4a2e234c1513.mp3","itemType":"pic","fontScale":0.8734375,"imgScale":0.19338422391857507,"mapScale":0.8734375,"gIdx":"0","useHand":true,"useBorder":true,"rect":{"x":773.66,"y":47.15,"width":76,"height":154.71}},{"id":"1621560091737","index":1,"audio_url":"http://staging-teach.cdn.ireadabc.com/8f73a6b73a217d25ca4f4a2e234c1513.mp3","itemType":"rect","fontScale":0.8734375,"imgScale":1,"mapScale":0.8734375,"gIdx":"1","useHand":true,"useBorder":true,"rect":{"x":150.66,"y":26.5,"width":200,"height":200}}]}]'
		const data = JSON.parse(dataJson);
		return data;
	},

	preloadItem() {
		this.addPreload()
		this.preload();
	},

	addPreload() {
		let item
		for (let i = 0; i < this.data.length; i++) {
			item = this.data[i]
			if (!item.bgItem) {
				this.data.splice(i, 1)
				continue
			}
			this._imageResList.push({ url: item.bgItem.url })
			for (let item of item.hotZoneItemArr) {
				item.pic_url && this._imageResList.push({ url: item.pic_url })
				item.audio_url && this._audioResList.push({ url: item.audio_url })
			}
		}
	},

	preload() {

		const preloadArr = this._imageResList.concat(this._audioResList).concat(this._animaResList);
		cc.assetManager.loadAny(preloadArr, null, null, (err, data) => {

			this.loadEnd();
			if (window && window["air"]) {
				window["air"].hideAirClassLoading();
			}

			cc.debug.setDisplayStats(false);
		});
	},


	loadEnd() {
		this.initData();
		this.initView();
		// this.initListener();
	},

	_cantouch: null,
	initData() {
		// 所有全局变量 默认都是null 
		this._cantouch = true;
	},

	initView() {
		this.initBg();
		this.initPic();
	},

	initBg() {
		const bgNode = cc.find('Canvas/bg');
		bgNode.scale = this._mapScaleMax;
		// this.picNode.scale = this._mapScaleMax
		// this.picTempNode.scale = this._mapScaleMax
		// console.log(this._mapScaleMax)
	},

	initPic() {
		this.picNode.opacity = 0
		cc.Tween.stopAllByTarget(this.picNode)
		cc.tween(this.picNode).to(1, { opacity: 255 }, { easing: 'cubicInOut' }).start()
		this.curPage = 0
		if (this.data.length == 1) {
			this.leftNode.active = false
			this.rightNode.active = false
		} else {
			this.leftNode.active = false
			this.rightNode.active = true
		}
		this.setContent(this.picNode, this.data[0])
	},
	curPage: null,

	onClickRight() {
		if (!this._cantouch) {
			return;
		}

		this.curPage += 1

		if (this.curPage == 1) {
			this.leftNode.active = true
			this.leftNode.scale = 1
		}

		if (this.curPage == this.data.length - 1) {
			this.rightNode.active = false
		}

		this.leftMove()

		cc.audioEngine.play(this.btnClip, false, 0.5)
	},

	onClickLeft() {
		if (!this._cantouch) {
			return;
		}
		this.curPage -= 1
		if (this.curPage == 0) {
			this.leftNode.active = false
		}

		if (this.curPage == this.data.length - 2) {
			this.rightNode.active = true
			this.rightNode.scale = 1
		}
		this.rightMove()

		cc.audioEngine.play(this.btnClip, false, 0.8)
	},



	/** 左移动 */
	leftMove() {
		this.stopAudio()
		this._cantouch = false;
		let width = cc.winSize.width + 64
		this.picNode.x = 0
		this.picTempNode.x = width
		this.setContent(this.picTempNode, this.data[this.curPage])
		cc.tween(this.picNode)
			.to(1, { x: -width }, { easing: 'cubicInOut' })
			.start();

		cc.tween(this.picTempNode)
			.to(1, { x: 0 }, { easing: 'cubicInOut' })
			.call(() => {
				let temp = this.picNode
				this.picNode = this.picTempNode
				this.picTempNode = temp
				this._cantouch = true;
			})
			.start();
	},

	/** 右移动 */
	rightMove() {
		this.stopAudio()
		this._cantouch = false;
		let width = cc.winSize.width + 64
		this.picNode.x = 0
		this.picTempNode.x = -width
		this.setContent(this.picTempNode, this.data[this.curPage])
		cc.tween(this.picNode)
			.to(1, { x: width }, { easing: 'cubicInOut' })
			.start();

		cc.tween(this.picTempNode)
			.to(1, { x: 0 }, { easing: 'cubicInOut' })
			.call(() => {
				let temp = this.picNode
				this.picNode = this.picTempNode
				this.picTempNode = temp
				this._cantouch = true
			})
			.start();
	},
	stopAudio() {
		cc.systemEvent.emit('stopMusic')
	},

	// ------------------------------------------------
	/**设置显示区域内容 */
	setContent(picNode, data) {
		if (!data) {
			console.log('数据错误')
			return
		}
		picNode.getComponent('picNode').initWithData(data)
	}

	// ------------------------------------------
});
